@@ -35,7 +35,7 @@ Branch: "TypeAlias" = tuple[Optional[Location], Location]
 Trace: "TypeAlias" = set[Branch]
 
 
-@lru_cache(maxsize=None)
+@functools.cache
 def should_trace_file(fname: str) -> bool:
     # fname.startswith("<") indicates runtime code-generation via compile,
     # e.g. compile("def ...", "<string>", "exec") in e.g. attrs methods.
@@ -47,29 +47,33 @@ def should_trace_file(fname: str) -> bool:
 # tool_id = 1 is designated for coverage, but we intentionally choose a
 # non-reserved tool id so we can co-exist with coverage tools.
 MONITORING_TOOL_ID = 3
-if sys.version_info[:2] >= (3, 12):
+if hasattr(sys, "monitoring"):
     MONITORING_EVENTS = {sys.monitoring.events.LINE: "trace_line"}
 
 
 class Tracer:
     """A super-simple branch coverage tracer."""
 
-    __slots__ = ("_previous_location", "_should_trace", "branches")
+    __slots__ = (
+        "_previous_location",
+        "_should_trace",
+        "_tried_and_failed_to_trace",
+        "branches",
+    )
 
     def __init__(self, *, should_trace: bool) -> None:
         self.branches: Trace = set()
         self._previous_location: Optional[Location] = None
+        self._tried_and_failed_to_trace = False
         self._should_trace = should_trace and self.can_trace()
 
     @staticmethod
     def can_trace() -> bool:
-        return (
-            (sys.version_info[:2] < (3, 12) and sys.gettrace() is None)
-            or (
-                sys.version_info[:2] >= (3, 12)
-                and sys.monitoring.get_tool(MONITORING_TOOL_ID) is None
-            )
-        ) and not PYPY
+        if PYPY:
+            return False
+        if hasattr(sys, "monitoring"):
+            return sys.monitoring.get_tool(MONITORING_TOOL_ID) is None
+        return sys.gettrace() is None
 
     def trace(self, frame, event, arg):
         try:
@@ -96,14 +100,23 @@ class Tracer:
         self._previous_location = current_location
 
     def __enter__(self):
+        self._tried_and_failed_to_trace = False
+
         if not self._should_trace:
             return self
 
-        if sys.version_info[:2] < (3, 12):
+        if not hasattr(sys, "monitoring"):
             sys.settrace(self.trace)
             return self
 
-        sys.monitoring.use_tool_id(MONITORING_TOOL_ID, "scrutineer")
+        try:
+            sys.monitoring.use_tool_id(MONITORING_TOOL_ID, "scrutineer")
+        except ValueError:
+            # another thread may have registered a tool for MONITORING_TOOL_ID
+            # since we checked in can_trace.
+            self._tried_and_failed_to_trace = True
+            return self
+
         for event, callback_name in MONITORING_EVENTS.items():
             sys.monitoring.set_events(MONITORING_TOOL_ID, event)
             callback = getattr(self, callback_name)
@@ -115,10 +128,13 @@ class Tracer:
         if not self._should_trace:
             return
 
-        if sys.version_info[:2] < (3, 12):
+        if not hasattr(sys, "monitoring"):
             sys.settrace(None)
             return
 
+        if self._tried_and_failed_to_trace:
+            return
+
         sys.monitoring.free_tool_id(MONITORING_TOOL_ID)
         for event in MONITORING_EVENTS:
             sys.monitoring.register_callback(MONITORING_TOOL_ID, event, None)
@@ -138,8 +154,9 @@ UNHELPFUL_LOCATIONS = (
     # Quite rarely, the first AFNP line is in Pytest's internals.
     "/_pytest/**",
     "/pluggy/_*.py",
-    # used by pytest for failure formatting in the terminal
-    "/pygments/lexer.py",
+    # used by pytest for failure formatting in the terminal.
+    # seen: pygments/lexer.py, pygments/formatters/, pygments/filter.py.
+    "/pygments/*",
     # used by pytest for failure formatting
     "/difflib.py",
     "/reprlib.py",
